"""
Constants that are used throughout the project.
"""
RESTORABLE_STATE_KEY = "--restorable-state"
SUBMIT_BUTTON_KEY = '--submit-button'
PREVIOUSLY_PRESSED_BUTTON = "--last-button"
LABEL_SEPARATOR = "$@~@$"
JSON_DECODE_SYMBOL = "$@JSON~@$"
